// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI20C500K/utils/util_channels_ready.c $
// $Rev: 53471 $
// $Date: 2023-07-26 15:49:04 -0500 (Wed, 26 Jul 2023) $

// 24DSI20C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi20c500k_channels_ready
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI20C500K_IOCTL_CHANNELS_READY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi20c500k_channels_ready(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Channels Ready", index);

	ret		= dsi20c500k_ioctl(fd, DSI20C500K_IOCTL_CHANNELS_READY, (void*) &set);
	errs	= ret ? 1 : 0;

	if (ret)
	{
		if (errno == ETIMEDOUT)
		{
			set	= DSI20C500K_CHANNELS_READY_NO;
		}
		else
		{
			errs++;
			printf("ERROR: dsi20c500k_ioctl() failure, errno = %d\n", errno);
		}
	}

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI20C500K_CHANNELS_READY_NO:

			ptr	= "No, timed out";
			break;

		case DSI20C500K_CHANNELS_READY_YES:

			ptr	= "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


